<?php

use \CuteLogger\Logger;

class Account
{
    private Database $db;

    public function __construct(Database $db)
    {
        $this->db = $db;
    }

    public function getCompletedJobsPercentage(int $user_id): int
    {
        $row = $this->db->setQuery("CALL spProjectsCompleted($user_id);")->loadResult();
        return $row ? $row->percentage_completed ?? 0 : 0;
    }

    public function Login(string $username, string $password): mixed
    {
        $user = $this->db->quote($username);
        $pass = $this->db->quote($password);
        return $this->db->setQuery("CALL spAccountLogin($user, $pass);")->loadResult();
    }

    public function Create(array $params): mixed
    {
        $first_name = $this->db->quote($params['first_name']);
        $last_name = $this->db->quote($params['last_name']);
        $email = $this->db->quote($params['email']);
        $password = $this->db->quote($params['password']);
        $account_type = $this->db->quote($params['account_type']);
        $country = $this->db->quote($params['country']);
        $client_id = $this->db->quote($params['client_id']);
        $secret_key = $this->db->quote($params['secret_key']);
        return $this->db->setQuery("CALL spAccountCreate($first_name, $last_name, $email, $password, $account_type, $country, $client_id, $secret_key);")->loadResult();
    }

    public function AddMoney(int $user_id, float $amount): bool
    {
        try {
            $user_id = $this->db->quote($user_id);
            $amount = $this->db->quote($amount);

            $this->db->setQuery("update tbprofile as a set a.`balance` = (a.`balance` + $amount) where a.`user_id`=$user_id;")->query();

        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    public function getTransferMethods(int $user_id): array|bool|null
    {
        return $this->db->select("*")->from("transfer_methods")->where("user_id=$user_id")->loadObject();
    }

    public function getTransferMethod(int $id, int $user_id)
    {
        return $this->db->setQuery("select * from `transfer_methods` where `transfer_method_id`=$id and `user_id`=$user_id")->loadResult();
    }

    public function setTransferMethod(stdClass $method): void
    {
        $this->db->insertObject('transfer_methods', $method);
    }

    public function setBilling(stdClass $billing)
    {
        $this->db->insertObject('tbbilling', $billing);
    }

    public function getBilling(int $user_id): mixed
    {
        return $this->db->select("*")->from("tbbilling")->where("user_id=$user_id")->loadResult();
    }

    public function getBidsDetails(int $user_id): mixed
    {
        $user_id = $this->db->quote($user_id);
        return $this->db->setQuery("SELECT
(select count(*) from tbbids as a where a.`user_id`=$user_id and a.`won`=1) as `bids_won`,
(select count(*) from tbbids as a where a.`user_id`=$user_id) as `jobs_applied`;")->loadResult();
    }

    public function AddSkills(int $category_id, array $skills): void
    {
        if (count($skills) > 0) {
            foreach ($skills as $skill) {
                $skill = $this->db->quote($skill);
                $this->db->setQuery("CALL spSkillsAdd($category_id, $skill)")->query();
            }
        }
    }

    public function AddService($id, $user_id, $title, $desc, $skills, $category, $hour_rate, $min_rate): bool
    {
        $id = $this->db->quote($id);
        $user_id = $this->db->quote($user_id);
        $title = $this->db->quote($title);
        $desc = $this->db->quote($desc);
        $skills = $this->db->quote($skills, false);
        $category = $this->db->quote($category);
        $hour_rate = $this->db->quote($hour_rate);
        $min_rate = $this->db->quote($min_rate);

        try {
            $this->db->setQuery("CALL spServiceAddUpdate($id, $user_id, $title, $desc, $skills, $category, $hour_rate, $min_rate)")->query();
        } catch (\Exception $e) {
            return false;
        }

        return true;
    }

    public function getBookmarks(int $user_id): bool|array|null
    {
        return $this->db
            ->select("b.title, a.created, c.status, a.project_id")
            ->from("bookmarks as a")
            ->join('inner', 'tbproject as b on(a.project_id=b.project_id)')
            ->join('inner', 'project_status as c on(c.project_id=b.project_id)')
            ->where("a.user_id=$user_id")
            ->loadObject();
    }

    public function getJobs(int $user_id): bool|array|null {
        return $this->db
            ->select("a.project_id, a.title, b.reject_reason, b.`status`, b.created, b.accept_date")
            ->from("tbproposal as b")
            ->join('inner', 'tbproject as a on(a.project_id=b.project_id)')
            //->join('inner', 'project_status as c on(c.project_id=b.project_id)')
            ->where("b.user_id=$user_id")
            ->loadObject();
    }

    public function getServices(int $user_id): mixed
    {
        $user_id = $this->db->quote($user_id);
        return $this->db->setQuery("select a.*, b.`name` as `category` from tbservices as a inner join tbcategory as b on(a.`category_id`=b.`category_id`) where a.`user_id`=$user_id order by a.`created` desc;")->loadObject();
    }

    public function getServiceById(int $id, int $user_id): mixed
    {
        $id = $this->db->quote($id);
        $user_id = $this->db->quote($user_id);
        return $this->db->setQuery("select * from tbservices as a where a.`service_id`=$id and a.`user_id`=$user_id;")->loadResult();
    }

    public function getById(int $user_id)
    {
        $user_id = $this->db->quote($user_id);
        return $this->db->setQuery("CALL spAccountById($user_id);")->loadResult();
    }

    public function getProfile(int $user_id)
    {
        $user_id = $this->db->quote($user_id);
        return $this->db->setQuery("CALL spProfileGet($user_id);")->loadResult();
    }

    public function getCompanyInfo(int $user_id){
        $user_id = $this->db->quote($user_id);
        return $this->db->select("*")->from("company")->where("user_id=$user_id")->loadResult();
    }

    public function GetByEmail(string $email): mixed
    {
        $user = $this->db->quote($email);
        return $this->db->setQuery("CALL spAccountByEmail($user);")->loadResult();
    }

    public function GetByClientId(string $client_id): mixed
    {
        $cid = $this->db->quote($client_id);
        return $this->db->setQuery("CALL spAccountByClientId($cid);")->loadResult();
    }

    public function GetEmailOptions(string $from, string $from_name, string $to, string $to_name, string $subject, string $body): array
    {
        return [
            'user' => $from,
            'from' => ['email' => $from, 'name' => $from_name],
            'recipients' => [
                ['email' => $to, 'name' => $to_name]
            ],
            'reply-to' => ['email' => $from, 'name' => $from_name],
            'subject' => $subject,
            'body' => $body
        ];
    }

    public function GetUserCode($user_id, $type): mixed
    {
        try {
            $user_id = $this->db->quote($user_id);
            $type = $this->db->quote($type);
            return $this->db->setQuery("CALL spUserCodeGet($user_id, $type);")->loadResult();
        } catch (\Exception $e) {
            error_log($e);
        }
        return null;
    }

    public function SetUserCode($user_id, $code): bool
    {
        try {
            $user_id = $this->db->quote($user_id);
            $code = $this->db->quote($code);
            $this->db->setQuery("CALL spUserCodeSet($user_id, $code, 'PASSWORD_RESET');")->query();
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }

    public function ChangePassword($user_id, $password): bool
    {
        try {
            $pass = $this->db->quote($password);
            $this->db->setQuery("CALL spUserPasswordUpdate($user_id, $pass);")->query();
            $this->db->insert("password_history")->columns(['user_id', 'password', 'created'])->values([$user_id, $password, gmdate("Y-m-d H:i:s")])->execute();
            return true;
        } catch (\Exception $e) {
            return false;
        }
    }

    public function getUserRating(int $user_id): stdClass
    {
        $rating = new stdClass();

        $data = $this->db->select("a.comment, sum(a.quality+a.communication+a.expertise+a.professionalism+a.work_with_again) as rating, count(a.user_rating_id) as count")
            ->from("tbuser_ratings as a")
            ->where("to_user_id=$user_id")
            ->loadResult();

        if($data && $data->count){
            $rating->rating = $data->rating / (5 * $data->count);
            $rating->percent = (($data->rating / ($data->count * 5)) / 5) * 100;
            $rating->count = $data->count;
        } else {
            $rating->rating = 0;
            $rating->percent = 0;
            $rating->count = 0;
        }

        return $rating;
    }

    public function getClientRating(int $user_id): stdClass
    {
        $rating = new stdClass();

        $data = $this->db->select("a.comment, sum(a.communication+a.professionalism+a.work_with_again) as rating, count(a.user_rating_id) as count")
            ->from("tbuser_ratings as a")
            ->where("to_user_id=$user_id")
            ->loadResult();

        if($data && $data->count){
            $rating->rating = $data->rating / (3 * $data->count);
            $rating->percent = (($data->rating / ($data->count * 3)) / 3) * 100;
            $rating->count = $data->count;
        } else {
            $rating->rating = 0;
            $rating->percent = 0;
            $rating->count = 0;
        }

        return $rating;
    }

    public function getRating(int $user_id): array
    {
        $rating = [];
        $rating['rating'] = 0;
        $rating['percent'] = 0;
        $rating['count'] = 0;
        $row = $this->db->setQuery("SELECT SUM(a.`rating`) AS rating, COUNT(a.`user_rating_id`) AS count FROM tbuser_ratings AS a WHERE a.`to_user_id`=$user_id;")->loadResult();
        if ($row) {
            if ($row->rating && $row->count) {
                $rating['rating'] = $row->rating / $row->count;
                $rating['percent'] = ($rating['rating'] * 2) * 10;
                $rating['count'] = $row->count;
            }
        }
        return $rating;
    }

    public function getReviewCount(int $user_id): int
    {
        return $this->db->setQuery("SELECT count(*) as reviews from tbuser_ratings as a where a.`ratee`=$user_id")->loadResult()->reviews;
    }

    /*public function getProjects(int $user_id, string $status, int $completed = 0): mixed{
        return $this->db->setQuery("SELECT a.id as pid, a.title, b.reject_reason, b.`status`, b.created, b.accept_date, c.first_name, c.last_name from tbproject as a inner join tbproposal as b on(a.id=b.project_id) inner join tbprofile as c on(a.user_id=c.user_id) where b.user_id=$user_id AND b.status IN ($status) AND a.completed=$completed;")->loadObject();
    }*/

    public function dashboard(int $user_id): mixed
    {
        $user_id = $this->db->quote($user_id);
        $query = "SELECT (select count(a.user_id) from tbproposal as a where a.user_id=$user_id AND a.awarded=1) AS bids_won,";
        $query .= "(SELECT COUNT(a.user_id) FROM tbproposal AS a WHERE a.user_id=$user_id) AS jobs_applied,";
        $query .= "(SELECT COUNT(a.ratee) FROM tbuser_ratings AS a WHERE a.ratee=$user_id) AS reviews_count";
        return $this->db->setQuery($query)->loadResult();
    }
}