<?php

namespace CuteLogger;

require_once __DIR__ . '/ILogger.php';
require_once __DIR__ . '/LogType.php';

use ILogger;
use LogType;

class Logger extends LogType implements ILogger
{
    private static string $app_id = 'PHP';
    private static string $log_path = "/srv/disk3/4124885/www/logs";

    private function __construct()
    {
    }

    public static function Trace(string $message): void
    {
        self::LogToFile($message, LogType::Trace);
    }

    public static function Warning(string $message): void
    {
        self::LogToFile($message, LogType::Warning);
    }

    public static function Debug(string $message): void
    {
        self::LogToFile($message, LogType::Debug);
    }

    public static function Success(string $message): void
    {
        self::LogToFile($message, LogType::Success);
    }

    public static function Error(string $message): void
    {
        self::LogToFile($message, LogType::Error);
    }

    public static function Exception(\Exception $e): void
    {
        self::LogToFile($e, LogType::Error);
    }

    private static function LogToFile(string $message, $type): void
    {
        $log = self::$log_path . '/' . $type . '-' . date('Y-m-d') . '.log';
        $data = '[' . date('d-M-Y H:i:s T') . '] ' . self::$app_id . ': ' . $message . PHP_EOL;
        self::write($log, $data);
    }

    private static function write($log, $data): void
    {
        @mkdir(self::$log_path, 0777, true);
        $fp = fopen($log, 'a+');
        fwrite($fp, $data);
        fclose($fp);
    }

}