<?php

class Date
{
    private function __construct()
    {
    }

    public static function format(DateTime|string $date, string $format = "Y-m-d H:i:s"): string
    {
        if ($date instanceof DateTime) {
            return $date->format($format);
        } else {
            return date($format, strtotime($date));
        }
    }

    public static function toTimezone($date, $tz = 'America/New_York', $format = 'Y-m-d H:i:s')
    {
        try {
            $datetime = new DateTime($date);
            $timezone = new DateTimeZone($tz);
            $datetime->setTimezone($timezone);
            return $datetime->format($format);
        } catch (Exception $e) {
        }
    }

    public static function toTimestamp(string $time, string $format = 'H:i'): int
    {
        //$date = DateTime::createFromFormat($format, $time, new DateTimeZone('UTC'));
        $date = DateTime::createFromFormat($format, $time);
        return $date->getTimestamp();
    }

    public static function fromTimestamp(int $timestamp, string $format = 'H:i'): string
    {
        return date($format, $timestamp);
    }

    /*public static function displayTime(int $timestamp): string
    {
        $ver = floatval(phpversion());
        if ($ver >= 8) {
            return gmdate("H:i:s", $timestamp);
        } else {
            $h = floor($timestamp / 3600);
            $m = $timestamp > 60 ? floor(($timestamp / 60) % 60) : 0;
            $s = floor($timestamp % 60);
            return sprintf("%02d:%02d:%02d", $h, $m, $s);
        }
    }*/

    public static function displayTime(int $timestamp): string
    {
        $h = floor($timestamp / 3600);
        $m = floor((int)($timestamp / 60) % 60);
        $s = floor($timestamp % 60);
        return sprintf("%02d:%02d:%02d", $h, $m, $s);
    }

    public static function diff(string $date1, string $date2 = 'now'): DateInterval
    {
        return date_diff(date_create($date1), date_create($date2));
    }

    public static function formatInterval(DateInterval $interval): string {

        $years = $interval->format("%y");
        $months = $interval->format("%m");
        $days = $interval->format("%a");
        $hours = $interval->format("%h");
        $minutes = $interval->format("%i");
        $seconds = $interval->format("%s");

        $string = $years > 0 ? "$years years ago" : null;
        if($string) { return $string; }
        $string = $months > 0 ? "$months months ago" : null;
        if($string) { return $string; }
        $string = $days > 0 ? "$days days ago" : null;
        if($string) { return $string; }
        $string = $hours > 0 ? "$hours hours ago" : null;
        if($string) { return $string; }
        $string = $minutes > 0 ? "$minutes minutes ago" : null;
        if($string) { return $string; }
        $string = $seconds > 0 ? "$seconds seconds ago" : null;
        if($string) { return $string; }

        return "";
    }

    public static function add(string $date, string $interval, string $format = 'Y-m-d'): string{
        //return date_format(date_add(date_create($date), date_interval_create_from_date_string($interval)), $format);
        return date_format(date_add(date_create($date), new DateInterval($interval)), $format);
    }

    public static function getOffset(string $time): int
    {
        $offset = -1;
        if ($data = explode(':', $time)) {
            $hour = $data[0];
            $min = $data[1];
            $ret = $hour + ($min / 60);
            $offset = ($ret * 3600) / 60;
        }
        return $offset;
    }

    public static function getNewDate(string $date, string $period): string
    {
        return ( new DateTime($date))->add(new DateInterval($period))->format("Y-m-d H:i:s");
    }

    public static function beforeToday(string $date): bool {
        return date_create($date) < date_create(date('Y-m-d H:i:s'));
    }

    public static function isValid(string $strDate, string $format = "Y-m-d"): bool {
        if(str_contains($strDate, '0000-00-00'))
            return false;

        $date = DateTime::createFromFormat($format, $strDate);
        return $date instanceof DateTime;
    }

}