<?php


class Paging
{
    private float $totalPages;
    private int $numPerPage;
    private float $currentPageNum;
    private array $pagesArray;
    private string $pager;

    /**
     * @param float $totalPages
     * @param int $numPerPage
     * @param string $pager
     */
    function __construct(float $totalPages, int $numPerPage = 5, string $pager = 'page')
    {
        if ($totalPages > 99999999999999) {
            //throw new Exception('Total pages cannot be greater than 99999999999999.');
            exit();
        }

        if (!is_float($totalPages)) {
            throw new Exception('Total pages must be a number.');
            exit();
        }

        $this->totalPages = $totalPages;
        $this->numPerPage = min($numPerPage, $totalPages);
        $this->currentPageNum = floatval($_GET[$pager] ?? 1);
        $this->pager = $pager;

        $this->getPages();
    }

    /**
     * @return void
     */
    private function getPages(): void
    {
        $this->getPagesArray();
    }

    /**
     * @return Paging
     */
    private function getPagesArray(): self
    {
        $half = floor($this->numPerPage / 2);
        if ($this->currentPageNum == 1) {
            $s = $this->currentPageNum;
            $e = $this->numPerPage + 1;
        } else {
            if ($this->currentPageNum > $half) {
                $s = $this->currentPageNum - $half;
            } else {
                $s = 1;
            }
            if ($this->currentPageNum <= $half) {
                $e = $this->numPerPage + 1;
            } else {
                if (($this->currentPageNum + $half) < $this->totalPages) {
                    $e = ($this->currentPageNum + 1) + $half;
                } else {
                    $s = $this->totalPages - ($this->numPerPage - 1);
                    $e = $this->totalPages + 1;
                }
            }
        }

        for ($i = $s; $i < $e; $i++) {
            $this->pagesArray[] = $i;
        }

        return $this;
    }

    /**
     * @return void
     */
    public function render(): void
    {
        if($this->totalPages > 0) {
            echo $this->buildPaging();
        } else {
            echo "<ul class='pagination'>";
            echo "<li class='page-item disabled'><span class='page-link'>Previous</span></li>";
            echo "<li class='page-item'><span style='cursor:default;' class='page-link'>1</span></li>";
            echo "<li class='page-item disabled'><span class='page-link'>Next</span></li>";
            echo "</ul>";
        }
    }

    /**
     * @return string
     */
    private function buildPaging(): string
    {
        $prev_disabled = $this->currentPageNum == 1 ? "disabled" : "";
        $next_disabled = ($this->currentPageNum >= $this->totalPages) ? "disabled" : "";

        $q = urlBuild($this->pager);
        $q = urlRemoveQueryString($q, $this->pager);
        $q = (urlHasQueryString($q)) ? $q . "&$this->pager=" : $q . "?$this->pager=";

        $paging = "<ul class='pagination'>";
        $paging .= "<li class='page-item $prev_disabled'>";
        if ($this->currentPageNum > 1) {
            $prev = $this->currentPageNum - 1;
            $link = "$q" . $prev;
            $paging .= "<a class='page-link' href='$link'>Previous</a></li>";
        } else {
            $paging .= "<a style='cursor:not-allowed;' class='page-link'>Previous</a>";
        }
        $paging .= "</li>";
        foreach ($this->pagesArray as $i) {
            $link = "$q" . $i;
            $active = ($this->currentPageNum == $i) ? 'active' : '';
            $paging .= "<li class='page-item $active'>";
            $paging .= ($this->currentPageNum == $i) ? "<span style='cursor:default;' class='page-link'>$i</span>" : "<a class='page-link' href='$link'>$i</a>";
            $paging .= "</li>";
        }
        $paging .= "<li class='page-item $next_disabled'>";
        if ($this->currentPageNum < $this->totalPages) {
            $next = $this->currentPageNum + 1;
            $link = "$q" . $next;
            $paging .= "<a class='page-link' href='$link'>Next</a></li>";
        } else {
            $paging .= "<a style='cursor:not-allowed;' class='page-link'>Next</a>";
        }
        $paging .= "</li>";
        $paging .= "</ul>";

        return $paging;
    }


}