<?php

use CuteHttp\Client;
use CuteLogger\Logger;

final class PayPal
{
    private static Database $db;
    private static string $client_id = 'AWhOemYULFG2sDOowTGZ9cVEmonJKa2kJv-36MIlUzIZNER499p1-YAuO_CrgjhbGAOGtHmqqGndos_-';
    private static string $secret = 'EOsAwRKz6io7cMqKxw5LdvzspJgY3hTbq7xdtGS3LgX22ODPGzV0VjLGzfJt4sS_qWLed64AeCH4Rehy';
    private static string $uri = 'https://api.sandbox.paypal.com';
    private static Client $client;
    private static string $environment = 'sandbox';
    private static string $contentType = 'application/json';
    private static ?string $scope;
    private static string $access_token;
    private static string $token_type;
    private static string $app_id;
    private static int $expires_in;
    private static string $nonce;
    private static self $instance;

    private function __construct()
    {
    }

    private function __clone()
    {
    }

    public static function getInstance(): self
    {
        if (!isset(self::$instance)) {
            self::$instance = new self();
        }

        self::init();

        return self::$instance;
    }

    private static function init(): void
    {
        self::$client = new Client(['base_uri' => self::$uri]);

        self::requestToken();
    }

    private static function requestToken(): void
    {
        $options = [
            'headers' => ['auth' => [self::$client_id, self::$secret]],
            'Content-Type' => 'application/x-www-form-urlencoded',
            'form_params' => ['grant_type' => 'client_credentials']
        ];
        try {
            $response = self::$client->request('POST', '/v1/oauth2/token', $options);
            if ($response) {
                self::assignToken(json_decode($response));
            }
        } catch (\Exception $e) {
            Logger::Exception($e);
        }
    }

    private static function assignToken($response): void
    {
        if (isset($response)) {
            self::$scope = $response->scope;
            self::$access_token = $response->access_token;
            self::$token_type = $response->token_type;
            self::$app_id = $response->app_id;
            self::$expires_in = $response->expires_in;
            self::$nonce = $response->nonce;
        }
    }

    public function getPaymentDetails(string $token): ?array
    {
        $options = [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token
            ]
        ];

        try {
            $request = self::$client->request('GET', "/v2/checkout/orders/$token", $options);
            $response = $request->getBody()->getContents();
            //Util::logProcessorData(self::PROCESSOR, "response: getPaymentDetails($token)", $response, Util::getUserIpAddress(), Util::getUserAgent());
            if ($response) {
                return json_decode($response, true);
            }
        } catch (\Exception $e) {
            Logger::Exception($e);
            return null;
        }
        return null;
    }

    private function getPaymentSource(): array
    {
        return ['payment_source' => [
            'paypal' => [
                'experience_context' => [
                    'payment_method_preference' => 'IMMEDIATE_PAYMENT_REQUIRED',
                    'user_action' => 'PAY_NOW',
                    'payment_method_selected' => 'PAYPAL',
                    'brand_name' => 'Workvee',
                    'locale' => 'en-US',
                    'return_url' => "https://alpha.workvee.com/emp?option=workspace&action=invoice-billing&pid=38&inv=9",
                    'cancel_url' => "https://alpha.workvee.com/emp?option=workspace&action=invoice-billing&pid=38&inv=9"
                ]
            ]
        ]
        ];
    }

    public function capturePayment(string $token, ?string $PayerID): array
    {
        $id = $PayerID;

        $options = [
            'headers' => [
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token,
            ]
        ];

        try {
            if (is_null($PayerID)) {
                $details = $this->getPaymentDetails($token);
                if ($details) {
                    $status = $details['status'];
                    $id = $details['id'];
                    return ['approved' => false, 'id' => $id, 'code' => $status];
                }
            } else {
                $request = self::$client->request('POST', "/v2/checkout/orders/$token/capture", $options);
                $response = $request->getBody()->getContents();
                //Util::logProcessorData(self::PROCESSOR, "response: capturePayment($token, $PayerID)", $response, Util::getUserIpAddress(), Util::getUserAgent());
                if ($response) {
                    $result = json_decode($response);
                    $status = $result->purchase_units[0]->payments->captures[0]->status;
                    $firstname = $result->payer->name->given_name;
                    $lastname = $result->payer->name->surname;
                    $id = $result->purchase_units[0]->payments->captures[0]->id;
                    $approved = strtoupper($status) === 'COMPLETED' || $this->isTestUser($firstname, $lastname);
                    return ['approved' => $approved, 'id' => $id, 'code' => 'APPROVED'];
                }
            }

        } catch (\Exception $e) {
            Logger::Exception($e);
            //AppLogger::log($e->getMessage(), ['approved' => false, 'id' => $id, 'code' => ''], AppLogger::WARNING);
            return ['approved' => false, 'id' => $id, 'code' => ''];
        }
        return ['approved' => false, 'id' => $id, 'code' => ''];
    }

    public function createOrder(): ?string
    {
        $url = null;

        $options = [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token,
                'PayPal-Request-Id' => uniqid(null, true)
            ],
            'body' => json_encode($this->getItems())
        ];

        try {
            $response = self::$client->request('POST', '/v2/checkout/orders', $options);
            //Util::logProcessorData(self::PROCESSOR, 'request: createOrder()', json_encode($options), Util::getUserIpAddress(), Util::getUserAgent());
            //$response = $request->getBody()->getContents();
            //Util::logProcessorData(self::PROCESSOR, 'response: createOrder()', $response, Util::getUserIpAddress(), Util::getUserAgent());
            if ($response) {
                $result = json_decode($response);
                if (strtoupper($result->status) === 'PAYER_ACTION_REQUIRED') {
                    foreach ($result->links as $link) {
                        if (strtolower($link->rel) === 'payer-action') {
                            $url = $link->href;
                            break;
                        }
                    }
                }
            }

        } catch (\Exception $e) {
            Logger::Exception($e);
        }

        return $url;
    }


    /******************************************/

    public function activatePlan(string $plan){

        Logger::Debug($plan);

        $body = "{
  'plan_id': 'P-1R80099192607563NMR7I3MI',
  'start_time': '2018-11-01T00:00:00Z',
  'quantity': '20',
  'shipping_amount': {
    'currency_code': 'USD',
    'value': '5.99'
  },
  'subscriber': {
    'name': {
      'given_name': 'John',
      'surname': 'Doe'
    },
    'email_address': 'customer@example.com',
    'shipping_address': {
      'name': {
        'full_name': 'John Doe'
      },
      'address': {
        'address_line_1': '2211 N First Street',
        'address_line_2': 'Building 17',
        'admin_area_2': 'San Jose',
        'admin_area_1': 'CA',
        'postal_code': '95131',
        'country_code': 'US'
      }
    }
  },
  'application_context': {
    'brand_name': 'walmart',
    'locale': 'en-US',
    'shipping_preference': 'SET_PROVIDED_ADDRESS',
    'user_action': 'SUBSCRIBE_NOW',
    'payment_method': {
      'payer_selected': 'PAYPAL',
      'payee_preferred': 'IMMEDIATE_PAYMENT_REQUIRED'
    },
    'return_url': 'https://example.com/returnUrl',
    'cancel_url': 'https://example.com/cancelUrl'
  }
}";

        $options = [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token,
                'PayPal-Request-Id' => uniqid(),
                'Prefer' => 'return=representation'
            ],
            'body' => json_encode([
                "plan_id" => "P-1R80099192607563NMR7I3MI"
            ])
        ];
        try{
            //$url = "https://www.sandbox.paypal.com/webapps/billing/plans/subscribe?plan_id=P-1R80099192607563NMR7I3MI";
            $response = self::$client->request('POST', "/v1/billing/subscriptions", $options);
            Logger::Debug($response);
        } catch (\Exception $e){
            Logger::Exception($e);
        }
    }

    public function createProduct(array $data): bool|string|null
    {
        $options = [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token,
                'PayPal-Request-Id' => uniqid()
            ],
            'body' => json_encode([
                'name' => $data['name'],
                'type' => 'SERVICE',
                'category' => 'SOFTWARE'
            ])
        ];

        try {
            $response = self::$client->request('POST', "/v1/catalogs/products", $options);
            Logger::Debug($response);
            return $response;
        } catch (\Exception $e){
            Logger::Exception($e);
        }

        return null;
    }

    public function createCheckoutButton(?string $reference_id = null, ?float $amount = null): string
    {
        $options = [
            'intent' => 'CAPTURE',
            'purchase_units' => [
                [
                    'invoice_id' => $reference_id ?? uniqid(),
                    'reference_id' => $reference_id ?? uniqid(),
                    'amount' => [
                        'currency_code' => 'USD',
                        'value' => number_format($amount ?? 1, 2)
                    ]
                ]
            ],
            'payment_source' => [
                'paypal' => [
                    'experience_context' => [
                        'payment_method_preference' => 'IMMEDIATE_PAYMENT_REQUIRED',
                        'user_action' => 'PAY_NOW',
                        'payment_method_selected' => 'PAYPAL',
                        'brand_name' => 'Workvee',
                        'locale' => 'en-US',
                        'return_url' => 'https://workvee.com/payment/finalize',
                        'cancel_url' => 'https://workvee.com/payment/finalize'
                    ]
                ]
            ]
        ];

        return json_encode(['purchase_units' => $options['purchase_units']]);
    }

    public function getPayer(string $token): ?array
    {
        $options = [
            'headers' => [
                'Accept' => 'application/json',
                'Content-Type' => 'application/json',
                'Authorization' => 'Bearer ' . self::$access_token
            ]
        ];

        try {
            $request = self::$client->request('GET', "/v2/checkout/orders/$token", $options);
            $response = $request->getBody()->getContents();
            //Util::logProcessorData(self::PROCESSOR, "response: getPayer($token)", $response, Util::getUserIpAddress(), Util::getUserAgent());
            if ($response) {
                $response = json_decode($response, true);
                $payer = $response['payment_source']['paypal'];
                $address = $response['purchase_units'][0]['shipping']['address'];
                return [
                    'firstname' => $payer['name']['given_name'],
                    'lastname' => $payer['name']['surname'],
                    'address1' => $address['address_line_1'],
                    'address2' => array_key_exists('address_line_2', $address) ? $address['address_line_2'] : '',
                    'zip' => $address['postal_code'],
                    'phone' => $payer['phone_number']['national_number'],
                    'email' => $payer['email_address']
                ];
            }
        } catch (\Exception $e) {
            Logger::Exception($e);
            //AppLogger::log($e->getMessage(), array("File" => $e->getFile()), AppLogger::WARNING);
            return null;
        }
        return null;
    }

    public function getOrderID(): string
    {
        return $_SESSION['PAYPAL_ORDER_ID'];
    }

    public function setOrderID(string $orderID): void
    {
        $_SESSION['PAYPAL_ORDER_ID'] = $orderID;
    }

    public function getPayerID(): string
    {
        return $_SESSION['PAYPAL_PAYER_ID'];
    }

    public function setPayerID(string $payerID): void
    {
        $_SESSION['PAYPAL_PAYER_ID'] = $payerID;
    }

    /***********************************************/

}


