<?php


class Project
{

    private Database $db;

    public function __construct(Database $db)
    {
        $this->db = $db;
    }

    public function get(int $user_id): mixed
    {
        $user_id = $this->db->quote($user_id);
        return $this->db->select("a.*, c.name as category, COUNT(b.proposal_id) AS num_of_proposals, a.status")
            ->from("tbproject AS a")
            ->join("inner", "tbcategory AS c")->on("(c.category_id=a.category_id)")
            ->join("left", "tbproposal AS b")->on("(a.project_id=b.project_id)")
            ->where("a.user_id=$user_id")
            ->group("a.project_id")
            ->order("num_of_proposals")
            ->loadObject();
    }

    public function byId(int $project_id): mixed
    {
        return $this->db->select("a.*, b.name as country, b.iso3 as abbr, c.name as category, d.name as company, d.city")
            ->from("tbproject as a")
            ->join('inner', "tbcategory as c on(c.category_id=a.category_id)")
            ->join('inner', "company as d on(d.user_id=a.user_id)")
            ->join('inner', "tbcountry as b on(b.name like d.country)")
            ->where("a.project_id=$project_id")
            ->order('a.created')
            ->loadResult();
    }

    public function getStatus(int $project_id, int $user_id): mixed{
        return $this->db->select("")
            ->from("")
            ->where("")
            ->loadResult();
    }

    public function getCandidates(int $user_id): mixed
    {
        return $this->db->setQuery("SELECT a.id AS proposal_id, a.hours_weekly, a.rate, a.due_num, a.due_val, a.created, a.rate, a.user_id, a.description as proposal, b.project_id, b.rate_type, c.email, d.first_name, d.last_name, d.phone, d.avatar, e.name AS country FROM tbproposal AS a INNER JOIN tbproject as b ON(a.project_id=b.project_id) INNER JOIN tbusers as c ON(a.user_id=c.id) INNER JOIN tbprofile as d ON(a.user_id=d.user_id) INNER JOIN tbcountry AS e ON(e.id=d.country_id) WHERE d.locked=0 AND b.user_id = $user_id group by c.email;")->loadObject();
    }

    public function getCandidatesByProject(int $project_id): mixed
    {
        $q = "SELECT a.proposal_id, a.created, a.user_id,";
        $q .= "a.description as proposal, b.project_id, b.rate_type, c.email, d.first_name, d.last_name,";
        $q .= "d.avatar, e.name AS country, d.tag_line,";
        $q .= "CASE WHEN d.company IS NOT NULL THEN d.company WHEN d.user_name IS NOT NULL THEN d.user_name ELSE CONCAT(d.first_name, ' ', d.last_name) END as name ";
        $q .= "FROM tbproposal AS a ";
        $q .= "INNER JOIN tbproject as b ON(a.project_id=b.project_id) ";
        $q .= "INNER JOIN tbusers as c ON(a.user_id=c.user_id) ";
        $q .= "INNER JOIN tbprofile as d ON(a.user_id=d.user_id) ";
        $q .= "INNER JOIN tbcountry AS e ON(e.iso3=d.country_id) ";
        //$q .= "INNER JOIN tbmilestones AS f ON(f.proposal_id=a.id) ";
        $q .= "WHERE d.locked=0 AND b.project_id = $project_id group by c.email;";
        return $this->db->setQuery("$q")->loadObject();
    }

    public function getAllCandidates(int $user_id): mixed
    {
        $q = "SELECT a.proposal_id, a.created, a.user_id,";
        $q .= "a.description as proposal, b.project_id, b.rate_type, c.email, d.first_name, d.last_name,";
        $q .= "d.avatar, e.name AS country, d.tag_line,";
        $q .= "CASE WHEN d.company IS NOT NULL THEN d.company WHEN d.user_name IS NOT NULL THEN d.user_name ELSE CONCAT(d.first_name, ' ', d.last_name) END as name ";
        $q .= "FROM tbproposal AS a ";
        $q .= "INNER JOIN tbproject as b ON(a.project_id=b.project_id) ";
        $q .= "INNER JOIN tbusers as c ON(a.user_id=c.user_id) ";
        $q .= "INNER JOIN tbprofile as d ON(a.user_id=d.user_id) ";
        $q .= "INNER JOIN tbcountry AS e ON(e.iso3=d.country_id) ";
        //$q .= "INNER JOIN tbmilestones AS f ON(f.proposal_id=a.id) ";
        $q .= "WHERE d.locked=0 AND b.user_id = $user_id group by c.email;";
        return $this->db->setQuery("$q")->loadObject();
    }

    public function getCandidateCount(int $project_id): int
    {
        return $this->db->setQuery("select count(*) as count from tbproposal as a where a.project_id = $project_id;")
            ->loadResult()
            ->count;
    }

    public function addAttachments(int $project_id, array $attachments): void
    {
        if (count($attachments) > 0) {

            $this->db->delete("tbproject_attachments")
                ->where("project_id=$project_id")
                ->execute();

            foreach ($attachments as $a) {
                $file_url = $a->url;
                $file_size = $a->size;
                $this->db->insert('tbproject_attachments')
                    ->columns(['project_id','file_url','file_size','created'])
                    ->values([$project_id, $file_url, $file_size, gmdate("Y-m-d H:i:s")])
                    ->execute();
            }
        }
    }

    public function getAttachments(int $project_id): array|bool|null
    {
        return $this->db->setQuery("SELECT * FROM tbproject_attachments AS a WHERE a.project_id=$project_id")->loadObject();
    }

    public function isAwarded(int $user_id, int $project_id): bool
    {
        $result = $this->db->setQuery("SELECT * FROM tbproposal AS a WHERE a.project_id=$project_id AND a.user_id=$user_id AND a.status >= 1;")->loadResult();
        return $result !== false;
    }

    public function getOwner(int $project_id){
        $user = $this->db->select('user_id')
            ->from('tbproject')
            ->where("project_id=$project_id")
            ->loadResult();

        return $this->db->setQuery("CALL spAccountById($user->user_id);")->loadResult();
    }

    public function isActive(int $project_id): bool{
        return true;
    }

    public function getHires(int $project_id): bool|array|null
    {
        return $this->db->select("a.user_id, CASE WHEN d.company IS NOT NULL THEN d.company WHEN d.user_name IS NOT NULL THEN d.user_name ELSE CONCAT(d.first_name, ' ', d.last_name) END as name")
            ->from("tbproposal as a")
            ->join("inner", "tbprofile as d on(d.user_id=a.user_id)")
            ->where("a.project_id=$project_id and a.status=2")
            ->loadObject();
    }

}